import java.net.*;
import java.io.*;

/**************************************************************************************
 *
 * Class UDPaudioProvider
 *
 **************************************************************************************/
public class UDPaudioProvider extends AudioProvider {
	
	static int port = 23657;				// client port
	static int serverPort = 23694;
	static int HEADER_LENGHT = 1;
	
	private byte[] header;
	private int maxAudioLen; 
	private byte[] AudioBuffer;
	private ADPCMdecoder decoder;

	DatagramPacket packet;
	DatagramSocket socket;
	InetAddress serverAddr;
	String serverName;
	
	AudioPlayBuffer player;

	public UDPaudioProvider(String hostname, int len) {
		super(len);

		maxAudioLen = lenght/2;
		AudioBuffer = new byte[maxAudioLen];
		header = new byte[1];
		decoder = new ADPCMdecoder(lenght);
		player = new AudioPlayBuffer();

		// initialize socket
		packet = new DatagramPacket(AudioBuffer, maxAudioLen);
		try {
			socket = new DatagramSocket(port);
		} catch (SocketException e) {
			System.err.println("UDPaudioProvider: " + e);
		}

		serverName = new String(hostname);
		initServer();
	}

	protected void finalize() {
		closeServer();
		socket.close();
	}

	private void initServer() {
		// initialize server
		try {
			serverAddr = InetAddress.getByName(serverName);
		} catch (UnknownHostException e) {
			System.err.println("UDPaudioProvider: " + e);
		}

		byte[] buf = new byte[100];
		buf[0] = 1; // ADD
		buf[1] = 0; // user name = ""
		
		try {
			String localname = InetAddress.getLocalHost().getHostName();
			localname.getBytes(0, localname.length(), buf, 21);
		} catch (IOException e) {
			System.err.println("UDPaudioProvider: " + e);
			System.exit(-1);
		}

		DatagramPacket pack = new DatagramPacket(buf, 71, serverAddr, serverPort);
		
		try {
			socket.send(pack);
		} catch (IOException e) {
			System.err.println("UDPaudioProvider: " + e);
		}
	}

	public void closeServer() {
		byte[] buf = new byte[100];
		buf[0] = 2; // DELETE
		buf[1] = 0; // user name = ""
		try {
			String localname = InetAddress.getLocalHost().getHostName();
			localname.getBytes(0, localname.length(), buf, 21);
		} catch (IOException e) {
			System.err.println("UDPaudioProvider: " + e);
		}

		DatagramPacket pack = new DatagramPacket(buf, 71, serverAddr, serverPort);
		
		try {
			socket.send(pack);
		} catch (IOException e) {
			System.err.println("UDPaudioProvider: " + e);
		}
	}
	
		
	public void run() {
		while(true) {  // condition to keep playing 
			putAudio();
			player.play(buffer);
		}
	}

	protected synchronized void putAudio() {
		try {
			// read from socket to AudioBuffer
			socket.receive(packet);

			decoder.putADPCM(AudioBuffer);
			decoder.decode();
			buffer = decoder.getBuffer();
			available = true;
		} catch (IOException e) {
			System.err.println("UDPaudioProvider: " + e);
		}
		notify();
	}
}
